macroScript rdg_mapTileTool
	category:"rdg::preset"
	toolTip:"adjust material tiling"
	buttonText:"rdg::MatTile"
(
	-- Dipl.-Des. Georg Duemlein
	-- http://www.preset.de
	-- ---
	-- 2007-02-20	added fixes
	--				added callbacks
	-- 2007-02-08	inital release
	
	struct __rdg_mapTileTool (
		theIniFile = getDir #plugcfg + "/rdg_tools.ini",
		structName = undefined,
		theRollout = undefined,
		theMaterialLUT = undefined,
		theObject = undefined,
		fn pickFilter obj = (
			obj.material != undefined
		),
		fn resetMaterialLUT = (
			theMaterialLUT = #()
			theRollout.coord_1.value = theRollout.coord_2.value = theRollout.coord_2.value = 1.
		),
		fn getTextures ref = (
			-- based on http://www.sloft.com/2005/11/26/maxscript-references/
		    local textureList = #()
		    local refList = refs.dependsOn ref
		    for r in refList do (
				if (superclassof r == texturemap) and (hasProperty r "coordinates") then (
						if classOf r.coordinates == StandardUVGen then (
							coordGen = 0
							coordDefault = [r.coordinates.u_tiling, r.coordinates.v_tiling, 0]
						)else(
							coordGen = 1
							coordDefault = r.coordinates.tiling
						)
						append textureList #(r, coordGen, coordDefault)
				)
		        join textureList (getTextures r)
		    )
		    textureList
		),
		fn moveMe pos= (
			setIniSetting theIniFile "mapTileTool" "Position" (pos as string)
		),
		fn openMe = (
			false
		),
		fn closeMe = (
			false	
		),
		fn enable = (
			theRollout.coord_1.enabled = theRollout.coord_2.enabled = theRollout.coord_3.enabled = true
			theRollout.apply_btn.enabled = theRollout.reset_btn.enabled = true
		),
		fn disable = (
			theRollout.obj_pick.text = "pick the Object"
			theRollout.coord_1.enabled = theRollout.coord_2.enabled = theRollout.coord_3.enabled = false
			theRollout.apply_btn.enabled = theRollout.reset_btn.enabled = false

		),
		fn updateTiling = (
			false
		),
		fn onPickObject obj =(
			theRollout.obj_pick.text = obj.name
			resetMaterialLUT()
			theMaterialLUT = getTextures obj.mat
			theObject = obj
			enable()
			
		),
		fn writeCoordinates definition = (
			local targetMat = definition[1]
			if definition[2] == 0 then (
				targetMat.coordinates.u_tiling = definition[3][1]
				targetMat.coordinates.v_tiling = definition[3][2]
			)else(
				targetMat.coordinates.tiling = definition[3]
			)
		),
		fn onCoordChanged val = (
			local theMultiplyer = [theRollout.coord_1.value, theRollout.coord_2.value, theRollout.coord_2.value] 
			local definition = #()
			for i = 1 to theMaterialLUT.count do(
				definition = deepcopy theMaterialLUT[i]
				definition[3] *= theMultiplyer
				writeCoordinates definition
			) 
		),
		fn followCallback = (
			theSel = selection as array
			if theSel.count == 1 and theSel[1].material != undefined then (
				rdg_mapTileTool.onPickObject theSel[1]
			)
			if theSel.count == 0 then (
				rdg_mapTileTool.disable()
			)
		),
		fn onFollowChanged val = (
			if val then (
				callbacks.addScript #selectionSetChanged "rdg_mapTileTool.followCallback()" id:#rdg_mapTileToolCB 
			)else(
				callbacks.removeScripts id:#rdg_mapTileToolCB
			)
		),	
		fn onReset = (
			-- print "reset"
			for i = 1 to theMaterialLUT.count do(
				-- format "before: %\n" theMaterialLUT[i] as string
				writeCoordinates theMaterialLUT[i]
				-- format "after: %\n" theMaterialLUT[i] as string
			)
			theRollout.coord_1.value = theRollout.coord_2.value = theRollout.coord_3.value = 1.0
		),
		fn onApply = (
			updateTiling()
			resetMaterialLUT()
			theMaterialLUT = getTextures theObject.mat
		),
		fn mkRollout name = (
			-- generate the rollout
			-- the interface
			rci = rolloutCreatorRDG name ("mapTileTool(rdg)")
			rci.begin()
			-- rci.openGroup "the Tiling Multiplier"
			rci.openGroup "the Object"
			rci.addControl #pickbutton "obj_pick" "pick the Object" paramStr:("filter:"+ structName + ".pickFilter width:140" )
			rci.closeGroup()
			rci.openGroup "the Tiling Multiplier"
			rci.addControl #spinner "coord_1" "u|x " paramStr:"range:[0.001, 99999., 1.] type:#float scale:0.1 enabled:false"
			rci.addControl #spinner "coord_2" "v|y " paramStr:"range:[0.001, 99999., 1.] type:#float scale:0.1 enabled:false"
			rci.addControl #spinner "coord_3" "w|z " paramStr:"range:[0.001, 99999., 1.] type:#float scale:0.1 enabled:false"
			rci.closeGroup()
			rci.openGroup "the Options"
			rci.addControl #checkbox "followSel" " follow selection " paramStr:""

			rci.closeGroup()
			rci.openGroup "the Actions"
			rci.addControl #button "apply_btn" "apply" paramStr:"enabled:false across:2"
			rci.addControl #button "reset_btn" "reset" paramStr:"enabled:false"
			rci.closeGroup()
			rci.addHandler "obj_pick" #picked paramStr:"obj" codeStr:(structName + ".onPickObject obj")
			rci.addHandler "coord_1" #changed paramStr:"val" codeStr:(structName + ".onCoordChanged val")
			rci.addHandler "coord_2" #changed paramStr:"val" codeStr:(structName + ".onCoordChanged val")
			rci.addHandler "coord_3" #changed paramStr:"val" codeStr:(structName + ".onCoordChanged val")
			rci.addHandler "followSel" #changed paramStr:"val" codeStr:(structName + ".onFollowChanged val")
			rci.addHandler "apply_btn" #pressed codeStr:(structName + ".onApply()")
			rci.addHandler "reset_btn" #pressed codeStr:(structName + ".onReset()")
			-- ---
			rci.addHandler name #open codeStr:(structName + ".openMe()")
			rci.addHandler name #moved paramStr:"arg" codeStr:(structName + ".moveMe arg")
			rci.addHandler name #close codeStr:(structName + ".closeMe()")
			return rci.end()
		),
		fn init structName_init= (
			-- init stuff
			-- --
			structName = structName_init
			theRollout_name = structName + "UI"
			theRollout = (mkRollout theRollout_name)
			thePos = execute (getIniSetting theIniFile "mapTileTool" "Position")
			if thePos == OK do thePos = [100,100]
			createDialog theRollout 160 240 thePos.x thePos.y
			-- ---			
		)

	)
	
	
	on execute do (
		global rdg_mapTileTool
		local theName = "rdg_mapTileTool"
		rdg_mapTileTool = __rdg_mapTileTool()
		try(
			destroyDialog (execute(theName + "UI"))
		)catch()	
		rdg_mapTileTool.init theName
	)
)
